@@ -30,15 +30,17 @@ class SdlSelectWidget : public SdlWidget
   public:
 	SdlSelectWidget(SDL_Renderer* renderer, std::string label, const SDL_FRect& rect);
 	SdlSelectWidget(SdlSelectWidget&& other) noexcept;
+	SdlSelectWidget(const SdlSelectWidget& other) = delete;
 	~SdlSelectWidget() override = default;
 
-	bool set_mouseover(SDL_Renderer* renderer, bool mouseOver);
-	bool set_highlight(SDL_Renderer* renderer, bool highlight);
-	bool update_text(SDL_Renderer* renderer);
+	SdlSelectWidget& operator=(const SdlSelectWidget& other) = delete;
+	SdlSelectWidget& operator=(SdlSelectWidget&& other) = delete;
+
+	virtual bool set_mouseover(SDL_Renderer* renderer, bool mouseOver);
+	virtual bool set_highlight(SDL_Renderer* renderer, bool highlight);
+	virtual bool update_text(SDL_Renderer* renderer);
 
   private:
-	SdlSelectWidget(const SdlSelectWidget& other) = delete;
-
 	std::string _text;
 	bool _mouseover;
 	bool _highlight;
