@@ -29,6 +29,7 @@
 
 #include <winpr/crt.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 
 #include "../core/settings.h"
 #include "../core/capabilities.h"
@@ -51,7 +52,8 @@ BOOL freerdp_addin_argv_add_argument_ex(ADDIN_ARGV* args, const char* argument,
 	if (len == 0)
 		len = strlen(argument);
 
-	new_argv = (char**)realloc(args->argv, sizeof(char*) * (args->argc + 1));
+	new_argv = (char**)realloc(
+	    (void*)args->argv, sizeof(char*) * (WINPR_ASSERTING_INT_CAST(uint32_t, args->argc) + 1));
 
 	if (!new_argv)
 		return FALSE;
@@ -81,8 +83,10 @@ BOOL freerdp_addin_argv_del_argument(ADDIN_ARGV* args, const char* argument)
 		if (strcmp(argument, arg) == 0)
 		{
 			free(arg);
-			memmove_s(&args->argv[x], (args->argc - x) * sizeof(char*), &args->argv[x + 1],
-			          (args->argc - x - 1) * sizeof(char*));
+			memmove_s((void*)&args->argv[x],
+			          (WINPR_ASSERTING_INT_CAST(uint32_t, args->argc - x)) * sizeof(char*),
+			          (void*)&args->argv[x + 1],
+			          (WINPR_ASSERTING_INT_CAST(uint32_t, args->argc - x - 1)) * sizeof(char*));
 			args->argv[args->argc - 1] = NULL;
 			args->argc--;
 			return TRUE;
@@ -154,7 +158,8 @@ int freerdp_addin_set_argument_value(ADDIN_ARGV* args, const char* option, const
 
 		if (p)
 		{
-			if (strncmp(args->argv[i], option, p - args->argv[i]) == 0)
+			if (strncmp(args->argv[i], option,
+			            WINPR_ASSERTING_INT_CAST(size_t, p - args->argv[i])) == 0)
 			{
 				free(args->argv[i]);
 				args->argv[i] = str;
@@ -221,13 +226,13 @@ BOOL freerdp_device_collection_add(rdpSettings* settings, RDPDR_DEVICE* device)
 			new_size = count * 2;
 
 		new_array =
-		    (RDPDR_DEVICE**)realloc(settings->DeviceArray, new_size * sizeof(RDPDR_DEVICE*));
+		    (RDPDR_DEVICE**)realloc((void*)settings->DeviceArray, new_size * sizeof(RDPDR_DEVICE*));
 
 		if (!new_array)
 			return FALSE;
 
 		settings->DeviceArray = new_array;
-		memset(&settings->DeviceArray[old], 0, (new_size - old) * sizeof(RDPDR_DEVICE*));
+		memset((void*)&settings->DeviceArray[old], 0, (new_size - old) * sizeof(RDPDR_DEVICE*));
 
 		if (!freerdp_settings_set_uint32(settings, FreeRDP_DeviceArraySize, new_size))
 			return FALSE;
@@ -300,7 +305,7 @@ RDPDR_DEVICE* freerdp_device_collection_find_type(rdpSettings* settings, UINT32
 	return NULL;
 }
 
-RDPDR_DEVICE* freerdp_device_new(UINT32 Type, size_t count, const char* args[])
+RDPDR_DEVICE* freerdp_device_new(UINT32 Type, size_t count, const char* const args[])
 {
 	size_t size = 0;
 	union
@@ -567,7 +572,7 @@ void freerdp_device_collection_free(rdpSettings* settings)
 			(void)freerdp_settings_set_pointer_array(settings, FreeRDP_DeviceArray, index, NULL);
 	}
 
-	free(settings->DeviceArray);
+	free((void*)settings->DeviceArray);
 
 	(void)freerdp_settings_set_pointer(settings, FreeRDP_DeviceArray, NULL);
 	(void)freerdp_settings_set_uint32(settings, FreeRDP_DeviceArraySize, 0);
@@ -588,10 +593,12 @@ BOOL freerdp_static_channel_collection_del(rdpSettings* settings, const char* na
 			if (strcmp(name, cur->argv[0]) == 0)
 			{
 				const size_t rem = settings->StaticChannelArraySize - count + 1;
-				memmove_s(&settings->StaticChannelArray[x], (count - x) * sizeof(ADDIN_ARGV*),
-				          &settings->StaticChannelArray[x + 1],
+				memmove_s((void*)&settings->StaticChannelArray[x],
+				          (count - x) * sizeof(ADDIN_ARGV*),
+				          (void*)&settings->StaticChannelArray[x + 1],
 				          (count - x - 1) * sizeof(ADDIN_ARGV*));
-				memset(&settings->StaticChannelArray[count - 1], 0, sizeof(ADDIN_ARGV*) * rem);
+				memset((void*)&settings->StaticChannelArray[count - 1], 0,
+				       sizeof(ADDIN_ARGV*) * rem);
 
 				freerdp_addin_argv_free(cur);
 				return freerdp_settings_set_uint32(settings, FreeRDP_StaticChannelCount, count - 1);
@@ -600,7 +607,7 @@ BOOL freerdp_static_channel_collection_del(rdpSettings* settings, const char* na
 	}
 	{
 		const size_t rem = settings->StaticChannelArraySize - count;
-		memset(&settings->StaticChannelArray[count], 0, sizeof(ADDIN_ARGV*) * rem);
+		memset((void*)&settings->StaticChannelArray[count], 0, sizeof(ADDIN_ARGV*) * rem);
 	}
 	return FALSE;
 }
@@ -622,8 +629,8 @@ BOOL freerdp_static_channel_collection_add(rdpSettings* settings, ADDIN_ARGV* ch
 		if (new_size == 0)
 			new_size = count * 2ul;
 
-		new_array =
-		    (ADDIN_ARGV**)realloc(settings->StaticChannelArray, new_size * sizeof(ADDIN_ARGV*));
+		new_array = (ADDIN_ARGV**)realloc((void*)settings->StaticChannelArray,
+		                                  new_size * sizeof(ADDIN_ARGV*));
 
 		if (!new_array)
 			return FALSE;
@@ -631,7 +638,7 @@ BOOL freerdp_static_channel_collection_add(rdpSettings* settings, ADDIN_ARGV* ch
 		settings->StaticChannelArray = new_array;
 		{
 			const size_t rem = new_size - oldSize;
-			memset(&settings->StaticChannelArray[oldSize], 0, sizeof(ADDIN_ARGV*) * rem);
+			memset((void*)&settings->StaticChannelArray[oldSize], 0, sizeof(ADDIN_ARGV*) * rem);
 		}
 		if (!freerdp_settings_set_uint32(settings, FreeRDP_StaticChannelArraySize, new_size))
 			return FALSE;
@@ -676,10 +683,10 @@ void freerdp_static_channel_collection_free(rdpSettings* settings)
 			freerdp_addin_argv_free(settings->StaticChannelArray[i]);
 	}
 
-	free(settings->StaticChannelArray);
-	freerdp_settings_set_uint32(settings, FreeRDP_StaticChannelArraySize, 0);
+	free((void*)settings->StaticChannelArray);
+	(void)freerdp_settings_set_uint32(settings, FreeRDP_StaticChannelArraySize, 0);
 	settings->StaticChannelArray = NULL;
-	freerdp_settings_set_uint32(settings, FreeRDP_StaticChannelCount, 0);
+	(void)freerdp_settings_set_uint32(settings, FreeRDP_StaticChannelCount, 0);
 }
 
 BOOL freerdp_dynamic_channel_collection_del(rdpSettings* settings, const char* name)
@@ -696,10 +703,12 @@ BOOL freerdp_dynamic_channel_collection_del(rdpSettings* settings, const char* n
 			if (strcmp(name, cur->argv[0]) == 0)
 			{
 				const size_t rem = settings->DynamicChannelArraySize - count + 1;
-				memmove_s(&settings->DynamicChannelArray[x], (count - x) * sizeof(ADDIN_ARGV*),
-				          &settings->DynamicChannelArray[x + 1],
+				memmove_s((void*)&settings->DynamicChannelArray[x],
+				          (count - x) * sizeof(ADDIN_ARGV*),
+				          (void*)&settings->DynamicChannelArray[x + 1],
 				          (count - x - 1) * sizeof(ADDIN_ARGV*));
-				memset(&settings->DynamicChannelArray[count - 1], 0, sizeof(ADDIN_ARGV*) * rem);
+				memset((void*)&settings->DynamicChannelArray[count - 1], 0,
+				       sizeof(ADDIN_ARGV*) * rem);
 
 				freerdp_addin_argv_free(cur);
 				return freerdp_settings_set_uint32(settings, FreeRDP_DynamicChannelCount,
@@ -728,7 +737,8 @@ BOOL freerdp_dynamic_channel_collection_add(rdpSettings* settings, ADDIN_ARGV* c
 		if (size == 0)
 			size = count * 2;
 
-		new_array = realloc(settings->DynamicChannelArray, sizeof(ADDIN_ARGV*) * size);
+		new_array =
+		    (ADDIN_ARGV**)realloc((void*)settings->DynamicChannelArray, sizeof(ADDIN_ARGV*) * size);
 
 		if (!new_array)
 			return FALSE;
@@ -736,7 +746,7 @@ BOOL freerdp_dynamic_channel_collection_add(rdpSettings* settings, ADDIN_ARGV* c
 		settings->DynamicChannelArray = new_array;
 		{
 			const size_t rem = size - oldSize;
-			memset(&settings->DynamicChannelArray[oldSize], 0, sizeof(ADDIN_ARGV*) * rem);
+			memset((void*)&settings->DynamicChannelArray[oldSize], 0, sizeof(ADDIN_ARGV*) * rem);
 		}
 		if (!freerdp_settings_set_uint32(settings, FreeRDP_DynamicChannelArraySize, size))
 			return FALSE;
@@ -773,22 +783,25 @@ void freerdp_addin_argv_free(ADDIN_ARGV* args)
 	{
 		for (int index = 0; index < args->argc; index++)
 			free(args->argv[index]);
-		free(args->argv);
+		free((void*)args->argv);
 	}
 
 	free(args);
 }
 
-ADDIN_ARGV* freerdp_addin_argv_new(size_t argc, const char* argv[])
+ADDIN_ARGV* freerdp_addin_argv_new(size_t argc, const char* const argv[])
 {
+	if (argc > INT32_MAX)
+		return NULL;
+
 	ADDIN_ARGV* args = calloc(1, sizeof(ADDIN_ARGV));
 	if (!args)
 		return NULL;
 	if (argc == 0)
 		return args;
 
-	args->argc = argc;
-	args->argv = calloc(argc, sizeof(char*));
+	args->argc = (int)argc;
+	args->argv = (char**)calloc(argc, sizeof(char*));
 	if (!args->argv)
 		goto fail;
 
@@ -804,7 +817,10 @@ ADDIN_ARGV* freerdp_addin_argv_new(size_t argc, const char* argv[])
 	return args;
 
 fail:
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
 	freerdp_addin_argv_free(args);
+	WINPR_PRAGMA_DIAG_POP
 	return NULL;
 }
 
@@ -818,7 +834,7 @@ ADDIN_ARGV* freerdp_addin_argv_clone(const ADDIN_ARGV* args)
 	if (!args)
 		return NULL;
 	cnv.c = args->argv;
-	return freerdp_addin_argv_new(args->argc, cnv.cc);
+	return freerdp_addin_argv_new(WINPR_ASSERTING_INT_CAST(uint32_t, args->argc), cnv.cc);
 }
 
 void freerdp_dynamic_channel_collection_free(rdpSettings* settings)
@@ -832,10 +848,10 @@ void freerdp_dynamic_channel_collection_free(rdpSettings* settings)
 			freerdp_addin_argv_free(settings->DynamicChannelArray[i]);
 	}
 
-	free(settings->DynamicChannelArray);
-	freerdp_settings_set_uint32(settings, FreeRDP_DynamicChannelArraySize, 0);
+	free((void*)settings->DynamicChannelArray);
+	(void)freerdp_settings_set_uint32(settings, FreeRDP_DynamicChannelArraySize, 0);
 	settings->DynamicChannelArray = NULL;
-	freerdp_settings_set_uint32(settings, FreeRDP_DynamicChannelCount, 0);
+	(void)freerdp_settings_set_uint32(settings, FreeRDP_DynamicChannelCount, 0);
 }
 
 void freerdp_capability_buffer_free(rdpSettings* settings)
@@ -855,7 +871,7 @@ void freerdp_capability_buffer_free(rdpSettings* settings)
 	free(settings->ReceivedCapabilityDataSizes);
 	settings->ReceivedCapabilityDataSizes = NULL;
 
-	free(settings->ReceivedCapabilityData);
+	free((void*)settings->ReceivedCapabilityData);
 	settings->ReceivedCapabilityData = NULL;
 	free(settings->ReceivedCapabilities);
 	settings->ReceivedCapabilities = NULL;
@@ -906,7 +922,7 @@ void freerdp_target_net_addresses_free(rdpSettings* settings)
 			free(settings->TargetNetAddresses[index]);
 	}
 
-	free(settings->TargetNetAddresses);
+	free((void*)settings->TargetNetAddresses);
 	free(settings->TargetNetPorts);
 	settings->TargetNetAddressCount = 0;
 	settings->TargetNetAddresses = NULL;
@@ -922,7 +938,7 @@ void freerdp_server_license_issuers_free(rdpSettings* settings)
 		for (UINT32 x = 0; x < settings->ServerLicenseProductIssuersCount; x++)
 			free(settings->ServerLicenseProductIssuers[x]);
 	}
-	free(settings->ServerLicenseProductIssuers);
+	free((void*)settings->ServerLicenseProductIssuers);
 	settings->ServerLicenseProductIssuers = NULL;
 	settings->ServerLicenseProductIssuersCount = 0;
 }
@@ -968,37 +984,41 @@ void freerdp_performance_flags_make(rdpSettings* settings)
 
 	if (freerdp_settings_get_bool(settings, FreeRDP_DisableThemes))
 		PerformanceFlags |= PERF_DISABLE_THEMING;
-	freerdp_settings_set_uint32(settings, FreeRDP_PerformanceFlags, PerformanceFlags);
+	(void)freerdp_settings_set_uint32(settings, FreeRDP_PerformanceFlags, PerformanceFlags);
 }
 
 void freerdp_performance_flags_split(rdpSettings* settings)
 {
-	freerdp_settings_set_bool(settings, FreeRDP_AllowFontSmoothing,
-	                          (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) &
-	                           PERF_ENABLE_FONT_SMOOTHING)
-	                              ? TRUE
-	                              : FALSE);
-	freerdp_settings_set_bool(settings, FreeRDP_AllowDesktopComposition,
-	                          (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) &
-	                           PERF_ENABLE_DESKTOP_COMPOSITION)
-	                              ? TRUE
-	                              : FALSE);
-	freerdp_settings_set_bool(
+	(void)freerdp_settings_set_bool(
+	    settings, FreeRDP_AllowFontSmoothing,
+	    (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) &
+	     PERF_ENABLE_FONT_SMOOTHING)
+	        ? TRUE
+	        : FALSE);
+	(void)freerdp_settings_set_bool(
+	    settings, FreeRDP_AllowDesktopComposition,
+	    (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) &
+	     PERF_ENABLE_DESKTOP_COMPOSITION)
+	        ? TRUE
+	        : FALSE);
+	(void)freerdp_settings_set_bool(
 	    settings, FreeRDP_DisableWallpaper,
 	    (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) & PERF_DISABLE_WALLPAPER)
 	        ? TRUE
 	        : FALSE);
-	freerdp_settings_set_bool(settings, FreeRDP_DisableFullWindowDrag,
-	                          (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) &
-	                           PERF_DISABLE_FULLWINDOWDRAG)
-	                              ? TRUE
-	                              : FALSE);
-	freerdp_settings_set_bool(settings, FreeRDP_DisableMenuAnims,
-	                          (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) &
-	                           PERF_DISABLE_MENUANIMATIONS)
-	                              ? TRUE
-	                              : FALSE);
-	freerdp_settings_set_bool(
+	(void)freerdp_settings_set_bool(
+	    settings, FreeRDP_DisableFullWindowDrag,
+	    (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) &
+	     PERF_DISABLE_FULLWINDOWDRAG)
+	        ? TRUE
+	        : FALSE);
+	(void)freerdp_settings_set_bool(
+	    settings, FreeRDP_DisableMenuAnims,
+	    (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) &
+	     PERF_DISABLE_MENUANIMATIONS)
+	        ? TRUE
+	        : FALSE);
+	(void)freerdp_settings_set_bool(
 	    settings, FreeRDP_DisableThemes,
 	    (freerdp_settings_get_uint32(settings, FreeRDP_PerformanceFlags) & PERF_DISABLE_THEMING)
 	        ? TRUE
@@ -1107,7 +1127,8 @@ int freerdp_set_param_uint64(rdpSettings* settings, int id, UINT64 param)
 
 char* freerdp_get_param_string(const rdpSettings* settings, int id)
 {
-	return (char*)freerdp_settings_get_string(settings, (FreeRDP_Settings_Keys_String)id);
+	const char* str = freerdp_settings_get_string(settings, (FreeRDP_Settings_Keys_String)id);
+	return WINPR_CAST_CONST_PTR_AWAY(str, char*);
 }
 
 int freerdp_set_param_string(rdpSettings* settings, int id, const char* param)
@@ -1199,47 +1220,66 @@ BOOL freerdp_settings_set_value_for_name(rdpSettings* settings, const char* name
 			    (_strnicmp(value, "FALSE", 6) == 0) || (_strnicmp(value, "OFF", 6) == 0);
 			if (!val && !nval)
 				return parsing_fail(name, "BOOL", value);
+
+			// NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
 			return freerdp_settings_set_bool(settings, (FreeRDP_Settings_Keys_Bool)index, val);
 		}
 		case RDP_SETTINGS_TYPE_UINT16:
 			if (!value_to_uint(value, &uval, 0, UINT16_MAX))
 				return parsing_fail(name, "UINT16", value);
-			if (!freerdp_settings_set_uint16(settings, (FreeRDP_Settings_Keys_UInt16)index, uval))
+
+			// NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
+			if (!freerdp_settings_set_uint16(settings, (FreeRDP_Settings_Keys_UInt16)index,
+			                                 (UINT16)uval))
 				return parsing_fail(name, "UINT16", value);
 			return TRUE;
 
 		case RDP_SETTINGS_TYPE_INT16:
 			if (!value_to_int(value, &ival, INT16_MIN, INT16_MAX))
 				return parsing_fail(name, "INT16", value);
-			if (!freerdp_settings_set_int16(settings, (FreeRDP_Settings_Keys_Int16)index, ival))
+
+			// NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
+			if (!freerdp_settings_set_int16(settings, (FreeRDP_Settings_Keys_Int16)index,
+			                                (INT16)ival))
 				return parsing_fail(name, "INT16", value);
 			return TRUE;
 		case RDP_SETTINGS_TYPE_UINT32:
 			if (!value_to_uint(value, &uval, 0, UINT32_MAX))
 				return parsing_fail(name, "UINT32", value);
-			if (!freerdp_settings_set_uint32(settings, (FreeRDP_Settings_Keys_UInt32)index, uval))
+
+			// NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
+			if (!freerdp_settings_set_uint32(settings, (FreeRDP_Settings_Keys_UInt32)index,
+			                                 (UINT32)uval))
 				return parsing_fail(name, "UINT32", value);
 			return TRUE;
 		case RDP_SETTINGS_TYPE_INT32:
 			if (!value_to_int(value, &ival, INT32_MIN, INT32_MAX))
 				return parsing_fail(name, "INT32", value);
-			if (!freerdp_settings_set_int32(settings, (FreeRDP_Settings_Keys_Int32)index, ival))
+
+			// NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
+			if (!freerdp_settings_set_int32(settings, (FreeRDP_Settings_Keys_Int32)index,
+			                                (INT32)ival))
 				return parsing_fail(name, "INT32", value);
 			return TRUE;
 		case RDP_SETTINGS_TYPE_UINT64:
 			if (!value_to_uint(value, &uval, 0, UINT64_MAX))
 				return parsing_fail(name, "UINT64", value);
+
+			// NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
 			if (!freerdp_settings_set_uint64(settings, (FreeRDP_Settings_Keys_UInt64)index, uval))
 				return parsing_fail(name, "UINT64", value);
 			return TRUE;
 		case RDP_SETTINGS_TYPE_INT64:
 			if (!value_to_int(value, &ival, INT64_MIN, INT64_MAX))
 				return parsing_fail(name, "INT64", value);
+
+			// NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
 			if (!freerdp_settings_set_int64(settings, (FreeRDP_Settings_Keys_Int64)index, ival))
 				return parsing_fail(name, "INT64", value);
 			return TRUE;
 
 		case RDP_SETTINGS_TYPE_STRING:
+			// NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
 			return freerdp_settings_set_string(settings, (FreeRDP_Settings_Keys_String)index,
 			                                   value);
 		case RDP_SETTINGS_TYPE_POINTER:
@@ -1251,7 +1291,8 @@ BOOL freerdp_settings_set_value_for_name(rdpSettings* settings, const char* name
 }
 
 BOOL freerdp_settings_set_pointer_len_(rdpSettings* settings, FreeRDP_Settings_Keys_Pointer id,
-                                       SSIZE_T lenId, const void* data, size_t len, size_t size)
+                                       FreeRDP_Settings_Keys_UInt32 lenId, const void* data,
+                                       size_t len, size_t size)
 {
 	BOOL rc = FALSE;
 	void* copy = NULL;
@@ -1259,12 +1300,14 @@ BOOL freerdp_settings_set_pointer_len_(rdpSettings* settings, FreeRDP_Settings_K
 	free(old);
 	if (!freerdp_settings_set_pointer(settings, id, NULL))
 		return FALSE;
-	if (lenId >= 0)
+	if (lenId != FreeRDP_UINT32_UNUSED)
 	{
-		if (!freerdp_settings_set_uint32(settings, (FreeRDP_Settings_Keys_UInt32)lenId, 0))
+		if (!freerdp_settings_set_uint32(settings, lenId, 0))
 			return FALSE;
 	}
 
+	if (len > UINT32_MAX)
+		return FALSE;
 	if (len == 0)
 		return TRUE;
 	copy = calloc(len, size);
@@ -1281,9 +1324,9 @@ BOOL freerdp_settings_set_pointer_len_(rdpSettings* settings, FreeRDP_Settings_K
 
 	// freerdp_settings_set_pointer takes ownership of copy
 	//  NOLINTNEXTLINE(clang-analyzer-unix.Malloc)
-	if (lenId < 0)
+	if (lenId == FreeRDP_UINT32_UNUSED)
 		return TRUE;
-	return freerdp_settings_set_uint32(settings, (FreeRDP_Settings_Keys_UInt32)lenId, len);
+	return freerdp_settings_set_uint32(settings, lenId, (UINT32)len);
 }
 
 const void* freerdp_settings_get_pointer(const rdpSettings* settings,
@@ -1424,10 +1467,10 @@ BOOL freerdp_settings_set_pointer_len(rdpSettings* settings, FreeRDP_Settings_Ke
 			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_MonitorDefArraySize,
 			                                         data, len, sizeof(rdpMonitor));
 		case FreeRDP_ClientAutoReconnectCookie:
-			return freerdp_settings_set_pointer_len_(settings, id, -1, data, len,
+			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_UINT32_UNUSED, data, len,
 			                                         sizeof(ARC_CS_PRIVATE_PACKET));
 		case FreeRDP_ServerAutoReconnectCookie:
-			return freerdp_settings_set_pointer_len_(settings, id, -1, data, len,
+			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_UINT32_UNUSED, data, len,
 			                                         sizeof(ARC_SC_PRIVATE_PACKET));
 		case FreeRDP_ClientTimeZone:
 			if (len > 1)
@@ -1435,7 +1478,7 @@ BOOL freerdp_settings_set_pointer_len(rdpSettings* settings, FreeRDP_Settings_Ke
 				WLog_ERR(TAG, "FreeRDP_ClientTimeZone::len must be 0 or 1");
 				return FALSE;
 			}
-			return freerdp_settings_set_pointer_len_(settings, id, -1, data, len,
+			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_UINT32_UNUSED, data, len,
 			                                         sizeof(TIME_ZONE_INFORMATION));
 		case FreeRDP_BitmapCacheV2CellInfo:
 			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_BitmapCacheV2NumCells,
@@ -1446,7 +1489,7 @@ BOOL freerdp_settings_set_pointer_len(rdpSettings* settings, FreeRDP_Settings_Ke
 				WLog_ERR(TAG, "FreeRDP_GlyphCache::len must be 0 or 10");
 				return FALSE;
 			}
-			return freerdp_settings_set_pointer_len_(settings, id, -1, data, len,
+			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_UINT32_UNUSED, data, len,
 			                                         sizeof(GLYPH_CACHE_DEFINITION));
 		case FreeRDP_FragCache:
 			if (len > 1)
@@ -1454,7 +1497,7 @@ BOOL freerdp_settings_set_pointer_len(rdpSettings* settings, FreeRDP_Settings_Ke
 				WLog_ERR(TAG, "FreeRDP_FragCache::len must be 0 or 1");
 				return FALSE;
 			}
-			return freerdp_settings_set_pointer_len_(settings, id, -1, data, len,
+			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_UINT32_UNUSED, data, len,
 			                                         sizeof(GLYPH_CACHE_DEFINITION));
 		case FreeRDP_StaticChannelArray:
 			if (data == NULL)
@@ -1477,7 +1520,8 @@ BOOL freerdp_settings_set_pointer_len(rdpSettings* settings, FreeRDP_Settings_Ke
 			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_ReceivedCapabilitiesSize,
 			                                         data, len, sizeof(char));
 		case FreeRDP_OrderSupport:
-			return freerdp_settings_set_pointer_len_(settings, id, -1, data, len, sizeof(char));
+			return freerdp_settings_set_pointer_len_(settings, id, FreeRDP_UINT32_UNUSED, data, len,
+			                                         sizeof(char));
 
 		case FreeRDP_MonitorIds:
 			return freerdp_settings_set_pointer_len_(
@@ -1741,7 +1785,7 @@ BOOL freerdp_settings_set_pointer_array(rdpSettings* settings, FreeRDP_Settings_
 				goto fail;
 			if ((offset >= maxOffset) || !data)
 				goto fail;
-			settings->OrderSupport[offset] = *(const BOOL*)data;
+			settings->OrderSupport[offset] = *(const BOOL*)data ? 1 : 0;
 			return TRUE;
 		case FreeRDP_GlyphCache:
 			maxOffset = 10;
@@ -1807,11 +1851,11 @@ UINT32 freerdp_settings_get_codecs_flags(const rdpSettings* settings)
 	UINT32 flags = FREERDP_CODEC_ALL;
 	if (settings->RemoteFxCodec == FALSE)
 	{
-		flags &= ~FREERDP_CODEC_REMOTEFX;
+		flags &= (uint32_t)~FREERDP_CODEC_REMOTEFX;
 	}
 	if (settings->NSCodec == FALSE)
 	{
-		flags &= ~FREERDP_CODEC_NSCODEC;
+		flags &= (uint32_t)~FREERDP_CODEC_NSCODEC;
 	}
 	/*TODO: check other codecs flags */
 	return flags;
@@ -1948,7 +1992,7 @@ BOOL freerdp_device_equal(const RDPDR_DEVICE* what, const RDPDR_DEVICE* expect)
 	return TRUE;
 }
 
-char* freerdp_rail_support_flags_to_string(UINT32 flags, char* buffer, size_t length)
+const char* freerdp_rail_support_flags_to_string(UINT32 flags, char* buffer, size_t length)
 {
 	const UINT32 mask =
 	    RAIL_LEVEL_SUPPORTED | RAIL_LEVEL_DOCKED_LANGBAR_SUPPORTED |
@@ -1993,14 +2037,16 @@ BOOL freerdp_settings_update_from_caps(rdpSettings* settings, const BYTE* capsFl
 	WINPR_ASSERT(capsSizes || (capsCount == 0));
 	WINPR_ASSERT(capsCount <= UINT16_MAX);
 
+	wLog* log = WLog_Get(TAG);
+
 	for (UINT32 x = 0; x < capsCount; x++)
 	{
 		if (capsFlags[x])
 		{
-			wStream buffer;
+			wStream buffer = { 0 };
 			wStream* sub = Stream_StaticConstInit(&buffer, capsData[x], capsSizes[x]);
 
-			if (!rdp_read_capability_set(sub, (UINT16)x, settings, serverReceivedCaps))
+			if (!rdp_read_capability_set(log, sub, (UINT16)x, settings, serverReceivedCaps))
 				return FALSE;
 		}
 	}
@@ -2213,3 +2259,114 @@ BOOL freerdp_settings_are_valid(const rdpSettings* settings)
 {
 	return settings != NULL;
 }
+
+/* Function to sort rdpMonitor arrays:
+ * 1. first element is primary monitor
+ * 2. all others are sorted by coordinates of x/y
+ */
+static int sort_monitor_fn(const void* pva, const void* pvb)
+{
+	const rdpMonitor* a = pva;
+	const rdpMonitor* b = pvb;
+	WINPR_ASSERT(a);
+	WINPR_ASSERT(b);
+	if (a->is_primary && b->is_primary)
+		return 0;
+	if (a->is_primary)
+		return -1;
+	if (b->is_primary)
+		return 1;
+
+	if (a->x != b->x)
+		return a->x - b->x;
+	if (a->y != b->y)
+		return a->y - b->y;
+	return 0;
+}
+
+BOOL freerdp_settings_set_monitor_def_array_sorted(rdpSettings* settings,
+                                                   const rdpMonitor* monitors, size_t count)
+{
+	WINPR_ASSERT(monitors || (count == 0));
+	if (count == 0)
+	{
+		if (!freerdp_settings_set_int32(settings, FreeRDP_MonitorLocalShiftX, 0))
+			return FALSE;
+		if (!freerdp_settings_set_int32(settings, FreeRDP_MonitorLocalShiftY, 0))
+			return FALSE;
+		if (!freerdp_settings_set_pointer_len(settings, FreeRDP_MonitorDefArray, NULL, 0))
+			return FALSE;
+		return freerdp_settings_set_uint32(settings, FreeRDP_MonitorCount, 0);
+		return TRUE;
+	}
+
+	// Find primary or alternatively the monitor at 0/0
+	const rdpMonitor* primary = NULL;
+	for (size_t x = 0; x < count; x++)
+	{
+		const rdpMonitor* cur = &monitors[x];
+		if (cur->is_primary)
+		{
+			primary = cur;
+			break;
+		}
+	}
+	if (!primary)
+	{
+		for (size_t x = 0; x < count; x++)
+		{
+			const rdpMonitor* cur = &monitors[x];
+			if ((cur->x == 0) && (cur->y == 0))
+			{
+				primary = cur;
+				break;
+			}
+		}
+	}
+
+	if (!primary)
+	{
+		WLog_ERR(TAG, "Could not find primary monitor, aborting");
+		return FALSE;
+	}
+
+	if (!freerdp_settings_set_pointer_len(settings, FreeRDP_MonitorDefArray, NULL, count))
+		return FALSE;
+	rdpMonitor* sorted = freerdp_settings_get_pointer_writable(settings, FreeRDP_MonitorDefArray);
+	WINPR_ASSERT(sorted);
+
+	size_t sortpos = 0;
+
+	/* Set primary. Ensure left/top is at 0/0 and flags contains MONITOR_PRIMARY */
+	sorted[sortpos] = *primary;
+	sorted[sortpos].x = 0;
+	sorted[sortpos].y = 0;
+	sorted[sortpos].is_primary = TRUE;
+	sortpos++;
+
+	/* Set monitor shift to original layout */
+	const INT32 offsetX = primary->x;
+	const INT32 offsetY = primary->y;
+	if (!freerdp_settings_set_int32(settings, FreeRDP_MonitorLocalShiftX, offsetX))
+		return FALSE;
+	if (!freerdp_settings_set_int32(settings, FreeRDP_MonitorLocalShiftY, offsetY))
+		return FALSE;
+
+	for (size_t x = 0; x < count; x++)
+	{
+		const rdpMonitor* cur = &monitors[x];
+		if (cur == primary)
+			continue;
+
+		rdpMonitor m = monitors[x];
+		m.x -= offsetX;
+		m.y -= offsetY;
+		sorted[sortpos++] = m;
+	}
+
+	// Sort remaining monitors by x/y ?
+	qsort(sorted, count, sizeof(rdpMonitor), sort_monitor_fn);
+
+	return freerdp_settings_set_uint32(settings, FreeRDP_MonitorCount,
+	                                   WINPR_ASSERTING_INT_CAST(uint32_t, count));
+}
