@@ -307,10 +307,7 @@ static INLINE char* base64_encode_ex(const BYTE* WINPR_RESTRICT alphabet,
                                      BOOL crLf, size_t lineSize)
 {
 	int c = 0;
-	const BYTE* q = NULL;
-	char* p = NULL;
-	char* ret = NULL;
-	int blocks = 0;
+	size_t blocks = 0;
 	size_t outLen = (length + 3) * 4 / 3;
 	size_t extra = 0;
 	if (crLf)
@@ -320,11 +317,13 @@ static INLINE char* base64_encode_ex(const BYTE* WINPR_RESTRICT alphabet,
 	}
 	size_t outCounter = 0;
 
-	q = data;
-	p = ret = (char*)malloc(outLen + extra + 1ull);
+	const BYTE* q = data;
+	BYTE* p = malloc(outLen + extra + 1ull);
 	if (!p)
 		return NULL;
 
+	char* ret = (char*)p;
+
 	/* b1, b2, b3 are input bytes
 	 *
 	 * 0         1         2
@@ -339,7 +338,7 @@ static INLINE char* base64_encode_ex(const BYTE* WINPR_RESTRICT alphabet,
 
 	/* first treat complete blocks */
 	blocks = length - (length % 3);
-	for (int i = 0; i < blocks; i += 3, q += 3)
+	for (size_t i = 0; i < blocks; i += 3, q += 3)
 	{
 		c = (q[0] << 16) + (q[1] << 8) + q[2];
 
@@ -379,6 +378,8 @@ static INLINE char* base64_encode_ex(const BYTE* WINPR_RESTRICT alphabet,
 			if (pad)
 				*p++ = '=';
 			break;
+		default:
+			break;
 	}
 
 	if (crLf && length % 3)
@@ -409,12 +410,9 @@ static INLINE void* base64_decode(const signed char* WINPR_RESTRICT alphabet,
                                   const char* WINPR_RESTRICT s, size_t length,
                                   size_t* WINPR_RESTRICT data_len, BOOL pad)
 {
-	int n[4];
-	BYTE* q = NULL;
+	int n[4] = { 0 };
 	BYTE* data = NULL;
-	size_t nBlocks = 0;
-	size_t outputLen = 0;
-	int remainder = length % 4;
+	const size_t remainder = length % 4;
 
 	if ((pad && remainder > 0) || (remainder == 1))
 		return NULL;
@@ -422,13 +420,13 @@ static INLINE void* base64_decode(const signed char* WINPR_RESTRICT alphabet,
 	if (!pad && remainder)
 		length += 4 - remainder;
 
-	q = data = (BYTE*)malloc(length / 4 * 3 + 1);
+	BYTE* q = data = (BYTE*)malloc(length / 4 * 3 + 1);
 	if (!q)
 		return NULL;
 
 	/* first treat complete blocks */
-	nBlocks = (length / 4);
-	outputLen = 0;
+	const size_t nBlocks = (length / 4);
+	size_t outputLen = 0;
 
 	if (nBlocks < 1)
 	{
@@ -446,9 +444,9 @@ static INLINE void* base64_decode(const signed char* WINPR_RESTRICT alphabet,
 		if ((n[0] == -1) || (n[1] == -1) || (n[2] == -1) || (n[3] == -1))
 			goto out_free;
 
-		q[0] = (n[0] << 2) + (n[1] >> 4);
-		q[1] = ((n[1] & 15) << 4) + (n[2] >> 2);
-		q[2] = ((n[2] & 3) << 6) + n[3];
+		q[0] = (BYTE)((n[0] << 2) + (n[1] >> 4));
+		q[1] = (BYTE)(((n[1] & 15) << 4) + (n[2] >> 2));
+		q[2] = (BYTE)(((n[2] & 3) << 6) + n[3]);
 		outputLen += 3;
 	}
 
@@ -461,7 +459,7 @@ static INLINE void* base64_decode(const signed char* WINPR_RESTRICT alphabet,
 	n[2] = remainder == 2 ? -1 : base64_decode_char(alphabet, *s++);
 	n[3] = remainder >= 2 ? -1 : base64_decode_char(alphabet, *s++);
 
-	q[0] = (n[0] << 2) + (n[1] >> 4);
+	q[0] = (BYTE)((n[0] << 2) + (n[1] >> 4));
 	if (n[2] == -1)
 	{
 		/* XX== */
@@ -469,22 +467,22 @@ static INLINE void* base64_decode(const signed char* WINPR_RESTRICT alphabet,
 		if (n[3] != -1)
 			goto out_free;
 
-		q[1] = ((n[1] & 15) << 4);
+		q[1] = (BYTE)((n[1] & 15) << 4);
 	}
 	else if (n[3] == -1)
 	{
 		/* yyy= */
 		outputLen += 2;
-		q[1] = ((n[1] & 15) << 4) + (n[2] >> 2);
-		q[2] = ((n[2] & 3) << 6);
+		q[1] = (BYTE)(((n[1] & 15) << 4) + (n[2] >> 2));
+		q[2] = (BYTE)((n[2] & 3) << 6);
 	}
 	else
 	{
 		/* XXXX */
 		outputLen += 3;
-		q[0] = (n[0] << 2) + (n[1] >> 4);
-		q[1] = ((n[1] & 15) << 4) + (n[2] >> 2);
-		q[2] = ((n[2] & 3) << 6) + n[3];
+		q[0] = (BYTE)((n[0] << 2) + (n[1] >> 4));
+		q[1] = (BYTE)(((n[1] & 15) << 4) + (n[2] >> 2));
+		q[2] = (BYTE)(((n[2] & 3) << 6) + n[3]);
 	}
 
 	if (data_len)
