@@ -56,6 +56,10 @@
 #define MIN(x, y) (((x) < (y)) ? (x) : (y))
 #endif
 
+static WINPR_FILE* pStdHandleFile = NULL;
+
+static void GetStdHandle_Uninit(void) __attribute__((destructor));
+
 static BOOL FileIsHandled(HANDLE handle)
 {
 	return WINPR_HANDLE_IS_HANDLED(handle, HANDLE_TYPE_FILE, FALSE);
@@ -71,13 +75,21 @@ static int FileGetFd(HANDLE handle)
 	return fileno(file->fp);
 }
 
-static BOOL FileCloseHandle(HANDLE handle)
+static BOOL FileCloseHandleInt(HANDLE handle, BOOL force)
 {
 	WINPR_FILE* file = (WINPR_FILE*)handle;
 
 	if (!FileIsHandled(handle))
 		return FALSE;
 
+	if (!force)
+	{
+		if (handle == pStdHandleFile)
+		{
+			return FALSE;
+		}
+	}
+
 	if (file->fp)
 	{
 		/* Don't close stdin/stdout/stderr */
@@ -93,6 +105,11 @@ static BOOL FileCloseHandle(HANDLE handle)
 	return TRUE;
 }
 
+static BOOL FileCloseHandle(HANDLE handle)
+{
+	return FileCloseHandleInt(handle, FALSE);
+}
+
 static BOOL FileSetEndOfFile(HANDLE hFile)
 {
 	WINPR_FILE* pFile = (WINPR_FILE*)hFile;
@@ -116,8 +133,10 @@ static BOOL FileSetEndOfFile(HANDLE hFile)
 	return TRUE;
 }
 
+// NOLINTBEGIN(readability-non-const-parameter)
 static DWORD FileSetFilePointer(HANDLE hFile, LONG lDistanceToMove,
                                 const PLONG lpDistanceToMoveHigh, DWORD dwMoveMethod)
+// NOLINTEND(readability-non-const-parameter)
 {
 	WINPR_FILE* pFile = (WINPR_FILE*)hFile;
 	INT64 offset = 0;
@@ -381,15 +400,16 @@ static BOOL FileGetFileInformationByHandle(HANDLE hFile,
 	lpFileInformation->ftLastAccessTime.dwLowDateTime = ft & 0xFFFFFFFF;
 	lpFileInformation->nFileSizeHigh = ((UINT64)st.st_size) >> 32ULL;
 	lpFileInformation->nFileSizeLow = st.st_size & 0xFFFFFFFF;
-	lpFileInformation->dwVolumeSerialNumber = st.st_dev;
-	lpFileInformation->nNumberOfLinks = st.st_nlink;
+	lpFileInformation->dwVolumeSerialNumber = (UINT32)st.st_dev;
+	lpFileInformation->nNumberOfLinks = (UINT32)st.st_nlink;
 	lpFileInformation->nFileIndexHigh = (st.st_ino >> 4) & 0xFFFFFFFF;
 	lpFileInformation->nFileIndexLow = st.st_ino & 0xFFFFFFFF;
 	return TRUE;
 }
 
-static BOOL FileLockFileEx(HANDLE hFile, DWORD dwFlags, DWORD dwReserved,
-                           DWORD nNumberOfBytesToLockLow, DWORD nNumberOfBytesToLockHigh,
+static BOOL FileLockFileEx(HANDLE hFile, DWORD dwFlags, WINPR_ATTR_UNUSED DWORD dwReserved,
+                           WINPR_ATTR_UNUSED DWORD nNumberOfBytesToLockLow,
+                           WINPR_ATTR_UNUSED DWORD nNumberOfBytesToLockHigh,
                            LPOVERLAPPED lpOverlapped)
 {
 #ifdef __sun
@@ -461,8 +481,10 @@ static BOOL FileLockFileEx(HANDLE hFile, DWORD dwFlags, DWORD dwReserved,
 	return TRUE;
 }
 
-static BOOL FileUnlockFile(HANDLE hFile, DWORD dwFileOffsetLow, DWORD dwFileOffsetHigh,
-                           DWORD nNumberOfBytesToUnlockLow, DWORD nNumberOfBytesToUnlockHigh)
+static BOOL FileUnlockFile(HANDLE hFile, WINPR_ATTR_UNUSED DWORD dwFileOffsetLow,
+                           WINPR_ATTR_UNUSED DWORD dwFileOffsetHigh,
+                           WINPR_ATTR_UNUSED DWORD nNumberOfBytesToUnlockLow,
+                           WINPR_ATTR_UNUSED DWORD nNumberOfBytesToUnlockHigh)
 {
 	WINPR_FILE* pFile = (WINPR_FILE*)hFile;
 #ifdef __sun
@@ -504,8 +526,10 @@ static BOOL FileUnlockFile(HANDLE hFile, DWORD dwFileOffsetLow, DWORD dwFileOffs
 	return TRUE;
 }
 
-static BOOL FileUnlockFileEx(HANDLE hFile, DWORD dwReserved, DWORD nNumberOfBytesToUnlockLow,
-                             DWORD nNumberOfBytesToUnlockHigh, LPOVERLAPPED lpOverlapped)
+static BOOL FileUnlockFileEx(HANDLE hFile, WINPR_ATTR_UNUSED DWORD dwReserved,
+                             WINPR_ATTR_UNUSED DWORD nNumberOfBytesToUnlockLow,
+                             WINPR_ATTR_UNUSED DWORD nNumberOfBytesToUnlockHigh,
+                             LPOVERLAPPED lpOverlapped)
 {
 	WINPR_FILE* pFile = (WINPR_FILE*)hFile;
 #ifdef __sun
@@ -553,10 +577,10 @@ static BOOL FileUnlockFileEx(HANDLE hFile, DWORD dwReserved, DWORD nNumberOfByte
 	return TRUE;
 }
 
-static UINT64 FileTimeToUS(const FILETIME* ft)
+static INT64 FileTimeToUS(const FILETIME* ft)
 {
-	const UINT64 EPOCH_DIFF_US = EPOCH_DIFF * 1000000ULL;
-	UINT64 tmp = ((UINT64)ft->dwHighDateTime) << 32 | ft->dwLowDateTime;
+	const INT64 EPOCH_DIFF_US = EPOCH_DIFF * 1000000LL;
+	INT64 tmp = ((INT64)ft->dwHighDateTime) << 32 | ft->dwLowDateTime;
 	tmp /= 10; /* 100ns steps to 1us step */
 	tmp -= EPOCH_DIFF_US;
 	return tmp;
@@ -566,14 +590,14 @@ static UINT64 FileTimeToUS(const FILETIME* ft)
 static struct timespec filetimeToTimespec(const FILETIME* ftime)
 {
 	WINPR_ASSERT(ftime);
-	UINT64 tmp = FileTimeToUS(ftime);
+	INT64 tmp = FileTimeToUS(ftime);
 	struct timespec ts = { 0 };
-	ts.tv_sec = tmp / 1000000ULL;
-	ts.tv_nsec = (tmp % 1000000ULL) * 1000ULL;
+	ts.tv_sec = tmp / 1000000LL;
+	ts.tv_nsec = (tmp % 1000000LL) * 1000LL;
 	return ts;
 }
 
-static BOOL FileSetFileTime(HANDLE hFile, const FILETIME* lpCreationTime,
+static BOOL FileSetFileTime(HANDLE hFile, WINPR_ATTR_UNUSED const FILETIME* lpCreationTime,
                             const FILETIME* lpLastAccessTime, const FILETIME* lpLastWriteTime)
 {
 	struct timespec times[2] = { { UTIME_OMIT, UTIME_OMIT },
@@ -648,8 +672,8 @@ static BOOL FileSetFileTime(HANDLE hFile, const FILETIME* lpCreationTime,
 
 	// TODO: Creation time can not be handled!
 	{
-		const int rc = utimes(pFile->lpFileName, timevals);
-		if (rc != 0)
+		const int res = utimes(pFile->lpFileName, timevals);
+		if (res != 0)
 			return FALSE;
 	}
 
@@ -961,7 +985,7 @@ static HANDLE FileCreateFileA(LPCSTR lpFileName, DWORD dwDesiredAccess, DWORD dw
 
 	if (fstat(fileno(pFile->fp), &st) == 0 && dwFlagsAndAttributes & FILE_ATTRIBUTE_READONLY)
 	{
-		st.st_mode &= ~(S_IWUSR | S_IWGRP | S_IWOTH);
+		st.st_mode &= WINPR_ASSERTING_INT_CAST(mode_t, ~(S_IWUSR | S_IWGRP | S_IWOTH));
 		fchmod(fileno(pFile->fp), st.st_mode);
 	}
 
@@ -969,14 +993,14 @@ static HANDLE FileCreateFileA(LPCSTR lpFileName, DWORD dwDesiredAccess, DWORD dw
 	return pFile;
 }
 
-static BOOL IsFileDevice(LPCTSTR lpDeviceName)
+static BOOL IsFileDevice(WINPR_ATTR_UNUSED LPCTSTR lpDeviceName)
 {
 	return TRUE;
 }
 
-static HANDLE_CREATOR FileHandleCreator = { IsFileDevice, FileCreateFileA };
+static const HANDLE_CREATOR FileHandleCreator = { IsFileDevice, FileCreateFileA };
 
-HANDLE_CREATOR* GetFileHandleCreator(void)
+const HANDLE_CREATOR* GetFileHandleCreator(void)
 {
 	return &FileHandleCreator;
 }
@@ -1001,10 +1025,14 @@ static WINPR_FILE* FileHandle_New(FILE* fp)
 	return pFile;
 }
 
+void GetStdHandle_Uninit(void)
+{
+	FileCloseHandleInt(pStdHandleFile, TRUE);
+}
+
 HANDLE GetStdHandle(DWORD nStdHandle)
 {
 	FILE* fp = NULL;
-	WINPR_FILE* pFile = NULL;
 
 	switch (nStdHandle)
 	{
@@ -1020,19 +1048,22 @@ HANDLE GetStdHandle(DWORD nStdHandle)
 		default:
 			return INVALID_HANDLE_VALUE;
 	}
-	pFile = FileHandle_New(fp);
-	if (!pFile)
+	if (!pStdHandleFile)
+		pStdHandleFile = FileHandle_New(fp);
+
+	if (!pStdHandleFile)
 		return INVALID_HANDLE_VALUE;
 
-	return (HANDLE)pFile;
+	return (HANDLE)pStdHandleFile;
 }
 
-BOOL SetStdHandle(DWORD nStdHandle, HANDLE hHandle)
+BOOL SetStdHandle(WINPR_ATTR_UNUSED DWORD nStdHandle, WINPR_ATTR_UNUSED HANDLE hHandle)
 {
 	return FALSE;
 }
 
-BOOL SetStdHandleEx(DWORD dwStdHandle, HANDLE hNewHandle, HANDLE* phOldHandle)
+BOOL SetStdHandleEx(WINPR_ATTR_UNUSED DWORD dwStdHandle, WINPR_ATTR_UNUSED HANDLE hNewHandle,
+                    WINPR_ATTR_UNUSED HANDLE* phOldHandle)
 {
 	return FALSE;
 }
@@ -1055,24 +1086,23 @@ BOOL GetDiskFreeSpaceA(LPCSTR lpRootPathName, LPDWORD lpSectorsPerCluster, LPDWO
 	return TRUE;
 }
 
-BOOL GetDiskFreeSpaceW(LPCWSTR lpwRootPathName, LPDWORD lpSectorsPerCluster,
+BOOL GetDiskFreeSpaceW(LPCWSTR lpRootPathName, LPDWORD lpSectorsPerCluster,
                        LPDWORD lpBytesPerSector, LPDWORD lpNumberOfFreeClusters,
                        LPDWORD lpTotalNumberOfClusters)
 {
-	LPSTR lpRootPathName = NULL;
 	BOOL ret = 0;
-	if (!lpwRootPathName)
+	if (!lpRootPathName)
 		return FALSE;
 
-	lpRootPathName = ConvertWCharToUtf8Alloc(lpwRootPathName, NULL);
-	if (!lpRootPathName)
+	char* rootPathName = ConvertWCharToUtf8Alloc(lpRootPathName, NULL);
+	if (!rootPathName)
 	{
 		SetLastError(ERROR_NOT_ENOUGH_MEMORY);
 		return FALSE;
 	}
-	ret = GetDiskFreeSpaceA(lpRootPathName, lpSectorsPerCluster, lpBytesPerSector,
+	ret = GetDiskFreeSpaceA(rootPathName, lpSectorsPerCluster, lpBytesPerSector,
 	                        lpNumberOfFreeClusters, lpTotalNumberOfClusters);
-	free(lpRootPathName);
+	free(rootPathName);
 	return ret;
 }
 
@@ -1343,7 +1373,7 @@ DWORD GetFullPathNameA(LPCSTR lpFileName, DWORD nBufferLength, LPSTR lpBuffer, L
 
 	dwStatus = GetFullPathNameW(lpFileNameW, nBufferLengthW, lpBufferW, &lpFilePartW);
 
-	ConvertWCharNToUtf8(lpBufferW, nBufferLengthW / sizeof(WCHAR), lpBuffer, nBufferLength);
+	(void)ConvertWCharNToUtf8(lpBufferW, nBufferLengthW / sizeof(WCHAR), lpBuffer, nBufferLength);
 
 	if (lpFilePart)
 		lpFilePart = lpBuffer + (lpFilePartW - lpBufferW);
@@ -1456,6 +1486,7 @@ HANDLE GetFileHandleForFileDescriptor(int fd)
 
 	(void)setvbuf(fp, NULL, _IONBF, 0);
 
+	// NOLINTNEXTLINE(clang-analyzer-unix.Stream)
 	pFile = FileHandle_New(fp);
 	if (!pFile)
 		return INVALID_HANDLE_VALUE;
